function [design, Estimates, cRKJP] = IKL(tau, nFinal, cvsqd, truemodel)

% Eta0 is MM, Eta1 is Exponential
% [design, Estimates, cRKJP] = IKL(.01, 40, .1, 0)

% cRKJP is the matrix [check, R, Kappa, J, orderedPoints]
% Estimates is the matrix [theta0hat, theta1hat, sigmahatsqd0, sigmahatsqd1]


x = linspace(.1,5,50); %Set the design space
N = length(x);
indices = 1:N;

check = [];
Theta1hat = [];
Theta0hat = [];
Sigmahatsqd0 = [];
Sigmahatsqd1 = [];
Kappa = [];
J = [];
R = [];

tau1 = tau;
tau0 = tau;

theta00 = [1,1];
theta10 = [1,1];

%% Sampling is currently done from the central distributions, not the
% closest pair

if truemodel == 0
    meany = eta0(x, theta00);
elseif truemodel == 1
    meany = eta1(x, theta10);
end


% Compute best fitting theta1, together with Eta1 and V1
%options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
%p1 = length(theta10);
%theta1 = lsqnonlin(@(theta) resid1(theta, eta1, x, meany'), theta10, [], [], options); 


% Compute best fitting theta0, together with Eta0 and V0
%p0 = length(theta00);
%theta0 = lsqnonlin(@(theta) resid0(theta, eta0, x, meany'), theta00, [], [], options); 

%% Set an initial design

% Set an intitial design of size nInit = 10; it is obtained as in Example 4 with
% E(Y|x) linear.

indPos = [2, 7, 12, 17, 22, 27, 32, 37, 42, 47]; 
%indPos = [2,3,4,5,6,7,8,9,10,12];
xi = zeros(N,1);
xi(indPos)=1;  %%This assumes no replicates
n = sum(xi);
xi = xi/n;
nInit = n;
orderedPoints = x(xi>0);  %% This vector will contain the design points, in the order in which they enter

%% Start the loop over design size
while n < nFinal
    L = [];
    
%% Sample; estimate parameters, hence Eta1, Eta0, V1, V0    

indPos = indices(xi>0)';
indZero = indices(xi==0)';

sigmasqd = log(1+cvsqd); % this is sigmasqd in the underlying normal dist'n, NOT var(Y).

% Sample:

if n == nInit
    xsample = x(indPos);
    alpha = log(meany/sqrt(1+cvsqd))';
    ysample = exp(alpha(indPos) + sqrt(sigmasqd)*randn(n,1))';
	theta1start = [1;1];
    theta0start = [1;1];
    
end    %% end of "if n == nInit"

% Estimate
options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
[theta1hat, resnorm1] = lsqnonlin(@(theta) resid1(theta, xsample, ysample), theta1start, [], [], options); 
[theta0hat, resnorm0] = lsqnonlin(@(theta) resid0(theta, xsample, ysample), theta0start, [], [], options); 

Theta1hat = [Theta1hat, theta1hat];
Theta0hat = [Theta0hat, theta0hat];

p1 = length(theta1hat); 
p0 = length(theta0hat);

sigmahatsqd0 = resnorm0/(n-p0);  
sigmahatsqd1 = resnorm1/(n-p1);
sigmahatsqd = [sigmahatsqd0,sigmahatsqd1];

Sigmahatsqd0 = [Sigmahatsqd0, sigmahatsqd0];
Sigmahatsqd1 = [Sigmahatsqd1, sigmahatsqd1];

theta1start = theta1hat;
theta0start = theta0hat;

% Form the estimated U and V matrices
  
Eta1 = eta1(x, theta1hat)';
U1 = eta1prime(x, theta1hat); % matrix of derivatives of eta1
[Q1,R1] = qr(U1);
V1 = Q1(:,(p1+1):N);

Eta0 = eta0(x, theta0hat)';
U0 = eta0prime(x, theta0hat);  % matrix of derivatives of eta0
[Q0,R0] = qr(U0);
V0 = Q0(:,(p0+1):N);

% Check:
Etad = Eta1 - Eta0;
check = [check, (norm(Etad) > (tau1 + tau0))];

xiPos = xi(indPos);  
Eta1Pos = Eta1(indPos);
Eta0Pos = Eta0(indPos);

if n == nInit

    %% Obtain the deltas for this initial design:    
    % First isolate the terms with xi>0

    Perm = eye(N);
    Perm = Perm([indPos;indZero],:);    
        %Perm*v permutes v, putting Pos terms first. 
        %The transpose Perm' returns the permuted vector to original order
    s = length(indPos);

    % These terms "...Pos" are the upper blocks, corresponding to xi>0:
    V1Pos = V1(indPos,:);
    V0Pos = V0(indPos,:);

    % These term "...Zero" are the upper blocks, corresponding to xi=0:
    V1Zero = V1(indZero,:);
    V0Zero = V0(indZero,:);

    %% Do the minimization over delta
    delta1 = tau1*ones(N,1)/sqrt(N);  
    delta0 = tau0*ones(N,1)/sqrt(N);
	delta1start = delta1(indPos);
	delta0start = delta0(indPos);    
       
    deltastart = [delta1start; delta0start];
    
    robust = 1;
    
    if robust==1
        options=optimset('Display','final','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
        [delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos, cvsqd), ...
            deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

        delta1Pos = delta(1:s);
        delta0Pos = delta((s+1):(2*s));
    else
    	delta1Pos = zeros(s,1);
        delta0Pos = zeros(s,1);
    end
 
    c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
    c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

    delta1Zero = V1Zero*c1star;
    delta0Zero = V0Zero*c0star;

    %% Return the elements of delta to their natural order
    delta1 = Perm'*[delta1Pos; delta1Zero];
    delta0 = Perm'*[delta0Pos; delta0Zero];

end   %% End of loop over "if n==nInit"

%% Form the a(x) ... d(x)  and get the next design point.
   
A0 = U0'*U0;
A1 = U1'*U1;
Dxi = diag(xi);
B0 = n*U0'*Dxi*U0;
B1 = n*U1'*Dxi*U1;
b0vec = n*U0'*Dxi*delta0;
b1vec = n*U1'*Dxi*delta1;
B0inv = inv(B0);
B1inv = inv(B1);

a0 = diag(U0*B0inv*A0*B0inv*U0');
a1 = diag(U1*B1inv*A1*B1inv*U1');

b0 = diag(U0*B0inv*U0');
b1 = diag(U1*B1inv*U1');

c0 = U0*B0inv*b0vec;
c1 = U1*B1inv*b1vec;

d0 = U0*B0inv*A0*B0inv*b0vec;
d1 = U1*B1inv*A1*B1inv*b1vec;

% Compute R, kappa, j

f1 = normpdf(log(ysample*sqrt(1+cvsqd)./eta1(xsample, theta1hat))/sqrt(sigmasqd));
f0 = normpdf(log(ysample*sqrt(1+cvsqd)./eta0(xsample, theta0hat))/sqrt(sigmasqd));

r = 2*sum(log(f1./f0));
R = [R, r];
pir = normcdf(r/(2*sqrt(abs(r))));
kappa = sqrt(abs(2*pir - 1));
if r < 0
    j = 0;
elseif r > 0
    j=1;
end

%kappa = 1;
%j=0;

Kappa = [Kappa, kappa];
J = [J, j];

%% Now do the minimization

cvec = [c0star; c1star];  % Here these are inputs, should they be optimized?
cvec = cvec(randperm(length(cvec)));  % Randomize them instead

L = Q(cvec, kappa, n, j, sigmahatsqd, Eta0, Eta1, V0, V1, a0, a1, b0, b1, c0, c1, d0, d1, cvsqd);
  
%% Form design
nextDesignPoint = x(L==max(L));
if length(nextDesignPoint) > 1
    rr = randperm(length(nextDesignPoint))
    nextDesignPoint = nextDesignPoint(rr(1));
end
xi = n*xi + (x==nextDesignPoint)';
nextIndex = indices(x==nextDesignPoint);

delta0(nextIndex) = V0(nextIndex,:)*c0star;  
delta1(nextIndex) = V1(nextIndex,:)*c1star;

n = round(sum(xi))
xi = xi/n;
orderedPoints = [orderedPoints nextDesignPoint];

if truemodel == 0
    newalpha = log(eta0(nextDesignPoint, theta00)/sqrt(1+cvsqd));
elseif truemodel == 1
    newalpha = log(eta1(nextDesignPoint, theta10)/sqrt(1+cvsqd));
end

nexty = exp(newalpha + sqrt(sigmasqd).*randn(1,1));
xsample = [xsample, nextDesignPoint];
ysample = [ysample, nexty];


%% End the loop over design size
end


%% Summarize the design
design = [x(xi>0); (xi(xi>0))'];

cRKJP = [check', R', Kappa', J', orderedPoints((nInit+1):nFinal)'];
Estimates = [Theta0hat', Theta1hat', Sigmahatsqd0', Sigmahatsqd1'];

%subplot(2,3,1)
%plot(x, eta0(x, theta00), x, eta1(x, theta10));
%xlim([0 5]);
%xlabel('true means')
%subplot(2,3,2)
%plot(x, eta0(x, theta0hat), x, eta1(x, theta1hat));
%xlim([0 5]);
%xlabel('estimated means')
subplot(1,3,1)
scatter(1:nFinal, xsample,10)
xlim([0, nFinal]);
axis([0 nFinal 0 5.2]);
xlabel('(a)')
subplot(1,3,2)
bar(x, xi)
axis([0 5.5 0 1]);
set(gca,'XTick',0:1:5)
xlabel('(b)')
subplot(1,3,3)
plot((nInit + 1):nFinal, Kappa, 'o', (nInit + 1):nFinal, J, '.')
axis([nInit nFinal -.25 1.25]);
set(gca,'YTick',0:.5:1)
xlabel('(c)')
%%
%% Subfunctions follow:
%%
%% ============================= subfunction "truemean" =============================
function truemean = truemean(x, truemodel, truetheta)
if truemodel==0;
    truemean = eta0(x,truetheta); 
elseif truemodel==1;
    truemean = eta1(x,truetheta); 
elseif truemodel==2;
    truemean = .35 + .12*x;
end

%% ============================= subfunction "eta1" =============================
function eta1 = eta1(x, theta) 
V = theta(1);
K = theta(2);
eta1 = V*(1-exp(-K*x)); % Expo
%eta1 = V*x./(K+x); % MM
%% ============================= subfunction "eta1prime" =============================
function eta1prime = eta1prime(x, theta) 
V = theta(1);
K = theta(2);
eta1prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
%eta1prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM

%% ============================= subfunction "resid1" =============================
function resid1 = resid1(theta, x, y) 
resid1 = y - eta1(x, theta);

%% ============================= subfunction "eta0" =============================
function eta0 = eta0(x, theta) 
V = theta(1);
K = theta(2);
%eta0 = V*(1-exp(-K*x)); % Expo
eta0 = V*x./(K+x); % MM

%% ============================= subfunction "eta0prime" =============================
function eta0prime = eta0prime(x, theta) 
V = theta(1);
K = theta(2);
%eta0prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
eta0prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM

%% ============================= subfunction "resid0" =============================
function resid0 = resid0(theta, x, y) 
resid0 = y - eta0(x, theta);

%% ============================= subfunction "mu" =============================
function mu = mu(Eta, delta) 
if min(Eta + delta <0)
   warning('log of negative number')
end
mu = max(1e-5, Eta + delta);
%mu = Eta + delta;

%% ============================= subfunction "div" =============================
function div = div(mu1, mu0, cvsqd)
div = ((log(mu1./mu0)).^2)/(2*log(1+cvsqd));

%% ============================= subfunction "P" =============================
function [P, g] = P(delta, xi, Eta1, Eta0, cvsqd)
s = length(xi);
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
P = xi'*div(mu1, mu0, cvsqd);

if nargout > 1
    qwe1 = (log(mu1./mu0))./mu1;
    qwe0 = (log(mu1./mu0))./mu0;
    g1 = 2*qwe1.*xi;
    g0 = -2*qwe0.*xi;
    g = [g1; g0];
end

%% ============================= subfunction "Pcon" =============================
function [c, ceq, GC, GCeq] = Pcon(delta, V1, V0, tau1, tau0)
ceq = 0;
s = length(delta)/2;
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
c = [delta1'*((V1*V1')\delta1)-tau1^2;delta0'*((V0*V0')\delta0)-tau0^2];
if nargout > 2
    GC = [2*((V1*V1')\delta1), zeros(s,1); zeros(s,1), 2*((V0*V0')\delta0)];
    GCeq = zeros(2*s,1);
end
  
%% ============================= subfunction "Q" =============================
% This is the vector of functions of c0vec, c1vec, each of which is (well, maybe not) to be
% minimized over these vectors.

function Q = Q(cvec, kappa, n, j, sigmahatsqd, Eta0, Eta1, V0, V1, a0, a1, b0, b1, c0, c1, d0, d1, cvsqd)

[N,Nmp0] = size(V0);
[N,Nmp1] = size(V1);
c0vec = cvec(1:Nmp0);
c1vec = cvec((Nmp0+1):(Nmp0+Nmp1));

delta0vec = V0*c0vec;
delta1vec = V1*c1vec;

mu0 = mu(Eta0, delta0vec);
mu1 = mu(Eta1, delta1vec);

if j==0
   t1 = sigmahatsqd(1)*a0 + 2*d0.*(c0-delta0vec)./(1+b0);
   t2 = a0.*((c0-delta0vec)./(1+b0)).^2;
   t3 = norm(c0vec)^2;
elseif j==1
   t1 = sigmahatsqd(2)*a1 + 2*d1.*(c1-delta1vec)./(1+b1);
   t2 = a1.*((c1-delta1vec)./(1+b1)).^2;
   t3 = norm(c1vec)^2;
end

term2 = t1 - t2 - t3;
term1 = div(mu1, mu0, cvsqd);
Q = (1-kappa)*term1/n + kappa*term2; 









