function [out, Power] = KLsequential(tau, nInit, nFinal, maxheight)

% [out, Power] = KLsequential(.01, 2, 20, [4,4,4,4]);

x = linspace(.1,5,50); %Set the design space
cvsqd = [1, .5, .1, .01];

for j = 1:length(cvsqd)    
        [theta1, theta0, check, xi, design, Dfinal, orderedPoints] = KL(x, tau, nInit, nFinal, cvsqd(j), 1);
        out(j).Des = design;
        out(j).check = check;
        subplot(2,2,j)
        bar(x,nFinal*xi);
        axis([0 5.1 0 maxheight(j)]);
        if j==1
            xlabel ('(a)');
        elseif j==2
            xlabel ('(b)'); 
        elseif j==3
            xlabel ('(c)');
        elseif j==4
            xlabel ('(d)'); 
        end
        multiple(j) = 1/(2*log(1 + cvsqd(j)));
        Power(j) = normcdf(sqrt(2*nFinal*Dfinal*multiple(j))-norminv(.9));
end



%% ============================= subfunction "KL" =============================
function [Theta1hat, Theta0hat, check, xi, design, Dfinal, orderedPoints] = KL(x, tau, nInit, nFinal, cvsqd, robust)

N = length(x);
indices = 1:N;
check = [];
Theta1hat = [];
Theta0hat = [];

tau1 = tau;
tau0 = tau;

%% Set an initial design

% Box-Lucas design for MM:
%xi = zeros(N,1);
%x1 = theta00(2)*x(N)/(x(N)+2*theta00(2));
%increm = (x(N)-x(1))/(N-1);
%index1 = 1+round((x1-x(1))/increm);  %%% CHECK THIS
%xi(index1) = nInit-floor(nInit/2);
%xi(N) = floor(nInit/2);

%%%%%%%%
xi = zeros(N,1);
x1 = .1;
x2 = 1.4;
increm = (x(N)-x(1))/(N-1);
index1 = 1+round((x1-x(1))/increm);  %%% CHECK THIS
xi(index1) = nInit-floor(nInit/2);
index2 = 1+round((x2-x(1))/increm);  %%% CHECK THIS
xi(index2) = nInit-floor(nInit/2);
%%%%%%%%

n = sum(xi)
xi = xi/n;
orderedPoints = [x1 x2];

%% Start the loop over design size
while n < nFinal
    
%% Sample; estimate parameters, hence Eta1, Eta0, V1, V0    

indPos = indices(xi>0)';
indZero = indices(xi==0)';

cv = sqrt(cvsqd);
sigmasqd = log(1+cv^2);

% Sample:

if n == nInit
    xsample = x(indPos);
    alpha = log(truemean(xsample)/sqrt(1+cv^2))';
    ysample = exp(alpha + sqrt(sigmasqd).*randn(n,1))';
	theta1start = [1;1];
    theta0start = [1;1];
else
    xsample = [xsample, nextDesignPoint];
    alpha = log(truemean(nextDesignPoint)/sqrt(1+cv^2))';
    nexty = exp(alpha + sqrt(sigmasqd).*randn(1,1));
    ysample = [ysample, nexty];
    theta1start = theta1hat;
    theta0start = theta0hat;
end


% Estimate
options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
theta1hat = lsqnonlin(@(theta) resid1(theta, xsample, ysample), theta1start, [], [], options); 
theta0hat = lsqnonlin(@(theta) resid0(theta, xsample, ysample), theta0start, [], [], options); 

Theta1hat = [Theta1hat, theta1hat];
Theta0hat = [Theta0hat, theta0hat];

p1 = length(theta1hat);   
Eta1 = eta1(x, theta1hat)';
Eta1prime = eta1prime(x, theta1hat); % matrix of derivatives of eta1
[Q1,R1] = qr(Eta1prime);
V1 = Q1(:,(p1+1):N);

p0 = length(theta0hat);
Eta0 = eta0(x, theta0hat)';
Eta0prime = eta0prime(x, theta0hat);  % matrix of derivatives of eta2
[Q0,R0] = qr(Eta0prime);
V0 = Q0(:,(p0+1):N);

% Check:
Etad = Eta1 - Eta0;
check = [check, norm(Etad) - (tau1 + tau0)];

%% Isolate the terms with xi>0

Perm = eye(N);
Perm = Perm([indPos;indZero],:);    
    %Perm*v permutes v, putting Pos terms first. 
	%The transpose Perm' returns the permuted vector to original order
s = length(indPos);

% These terms "...Pos" are the upper blocks, corresponding to xi>0:
xiPos = xi(indPos);
Eta1Pos = Eta1(indPos);
Eta0Pos = Eta0(indPos);
V1Pos = V1(indPos,:);
V0Pos = V0(indPos,:);

% These term "...Zero" are the upper blocks, corresponding to xi=0:
V1Zero = V1(indZero,:);
V0Zero = V0(indZero,:);

%% Do the minimization over delta

if n > nInit 
    newIndex = indices(obj==max(obj));
    delta1(newIndex) = 0;
    delta1start = delta1(indPos);
    delta0(newIndex) = 0;
    delta0start = delta0(indPos);    
else
    delta1start = zeros(s,1);
    delta0start = zeros(s,1);
end
deltastart = [delta1start; delta0start];

if robust==1
    options=optimset('Display','final','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
    [delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos), ...
        deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

    delta1Pos = delta(1:s);
    delta0Pos = delta((s+1):(2*s));
else
	delta1Pos = zeros(s,1);
    delta0Pos = zeros(s,1);
end
 
c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

delta1Zero = V1Zero*c1star;
delta0Zero = V0Zero*c0star;

%% Return the elements of delta to their natural order
delta1 = Perm'*[delta1Pos; delta1Zero];
delta0 = Perm'*[delta0Pos; delta0Zero];

%% Obtain the next design
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
obj = div(mu1, mu0);
%plot(x,obj); keyboard


%% Form design
nextDesignPoint = x(obj==max(obj));
if length(nextDesignPoint) > 1
    r = randperm(length(nextDesignPoint));
    nextDesignPoint = nextDesignPoint(r(1));
end
xi = n*xi + (x==nextDesignPoint)';
n = sum(xi)
xi = xi/n;
orderedPoints = [orderedPoints nextDesignPoint];

%% End the loop over design size
end

%% Do one more minimization

if n > nInit 
    newIndex = indices(obj==max(obj));
    delta1(newIndex) = 0;
    delta1start = delta1(indPos);
    delta0(newIndex) = 0;
    delta0start = delta0(indPos);    
else
    delta1start = zeros(s,1);
    delta0start = zeros(s,1);
end
deltastart = [delta1start; delta0start];

if robust==1
    options=optimset('Display','final','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
    [delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos), ...
        deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

    delta1Pos = delta(1:s);
    delta0Pos = delta((s+1):(2*s));
else
	delta1Pos = zeros(s,1);
    delta0Pos = zeros(s,1);
end
 
c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

delta1Zero = V1Zero*c1star;
delta0Zero = V0Zero*c0star;

%% Return the elements of delta to their natural order
delta1 = Perm'*[delta1Pos; delta1Zero];
delta0 = Perm'*[delta0Pos; delta0Zero];

mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
obj = div(mu1, mu0);

%% Summarize the design
design = [x(xi>0); (xi(xi>0))'];
Dfinal = xi'*obj; 

%%
%% Subfunctions follow:
%%
%% ============================= subfunction "truemean" =============================
function truemean = truemean(x)
truemean = .35 + .12*x;

%% ============================= subfunction "eta1" =============================
function eta1 = eta1(x, theta) 
V = theta(1);
K = theta(2);
eta1 = V*(1-exp(-K*x)); % Expo
%eta1 = V*x./(K+x); % MM
%% ============================= subfunction "eta1prime" =============================
function eta1prime = eta1prime(x, theta) 
V = theta(1);
K = theta(2);
eta1prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
%eta1prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM

%% ============================= subfunction "resid1" =============================
function resid1 = resid1(theta, x, y) 
resid1 = y - eta1(x, theta);

%% ============================= subfunction "eta0" =============================
function eta0 = eta0(x, theta) 
V = theta(1);
K = theta(2);
%eta0 = V*(1-exp(-K*x)); % Expo
eta0 = V*x./(K+x); % MM

%% ============================= subfunction "eta0prime" =============================
function eta0prime = eta0prime(x, theta) 
V = theta(1);
K = theta(2);
%eta0prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
eta0prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM

%% ============================= subfunction "resid0" =============================
function resid0 = resid0(theta, x, y) 
resid0 = y - eta0(x, theta);

%% ============================= subfunction "mu" =============================
function mu = mu(Eta, delta) 
if min(Eta + delta <0)
   warning('log of negative number')
end
mu = max(1e-5, Eta + delta);
%mu = Eta + delta;

%% ============================= subfunction "div" =============================
function div = div(mu1, mu0)
div = (log(mu1./mu0)).^2;

%% ============================= subfunction "P" =============================
function [P, g] = P(delta, xi, Eta1, Eta0)
s = length(xi);
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
P = xi'*div(mu1, mu0);

if nargout > 1
    qwe1 = (log(mu1./mu0))./mu1;
    qwe0 = (log(mu1./mu0))./mu0;
    g1 = 2*qwe1.*xi;
    g0 = -2*qwe0.*xi;
    g = [g1; g0];
end

%% ============================= subfunction "Pcon" =============================
function [c, ceq, GC, GCeq] = Pcon(delta, V1, V0, tau1, tau0)
ceq = 0;
s = length(delta)/2;
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
c = [delta1'*((V1*V1')\delta1)-tau1^2;delta0'*((V0*V0')\delta0)-tau0^2];
if nargout > 2
    GC = [2*((V1*V1')\delta1), zeros(s,1); zeros(s,1), 2*((V0*V0')\delta0)];
    GCeq = zeros(2*s,1);
end
  
    
