 
function [Dmat, theta0, theta1, design, check, Dfinal, Power, maxPower] = ...
    KLstatic(eta01T, tau, S, n, K, alg)

% Example 3.  Exponential mean, alg = 2:
% [Dmat, theta0, theta1, design, check, Dfinal, Power, maxPower] = KLstatic(['M', 'E', 'E'], .05, [.1, 5, 50], 20, 500, 2)
% For Linear mean use ['M', 'E', 'L']

% eta01T is a triple, [eta_0, eta_1, eta_true] coded by 'M' for MM, 
    % 'E' for exponential, 'L' for linear.  
% S is the design space - an interval [S(1), S(2)] from which N = S(3) 
    %equally spaced points are taken.   

tau1 = tau;
tau0 = tau;
N = S(3);
x = linspace(S(1),S(2),N); %Set the design space
indices = 1:N;

if(eta01T(1) == 'M') 
    eta0 = @mm;
    eta0prime = @mmprime;
elseif(eta01T(1) == 'E')
    eta0 = @expo;
    eta0prime = @expoprime;
end

if(eta01T(2) == 'M') 
    eta1 = @mm;
    eta1prime = @mmprime;
elseif(eta01T(2) == 'E')
    eta1 = @expo;
    eta1prime = @expoprime;
end

%% Set up those terms which are only computed once:

theta10 = [1;1];
theta00 = [1;1];

% Simulate E(Y|x)

if(eta01T(3) == 'M')
    meany = mm(x, theta00)';
elseif(eta01T(3) == 'E')
    meany = expo(x, theta10)';
elseif(eta01T(3) == 'L')
    X = [ones(N,1),x'];
    lincoefs = regress(eta0(x,theta00)',X);
    meany = X*lincoefs;
end

%figure; plot(x, meany, 'k', x, eta0(x,theta00)', ':', x, eta1(x,theta10)','--'); keyboard;


% Compute best fitting theta1, together with Eta1 and V1
options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
p1 = length(theta10);
theta1 = lsqnonlin(@(theta) resid1(theta, eta1, x, meany'), theta10, [], [], options); 
Eta1 = eta1(x, theta1)'; % vector of eta1 values
Eta1prime = eta1prime(x, theta1); % matrix of derivatives of eta1
[Q1,R1] = qr(Eta1prime);
V1 = Q1(:,(p1+1):N);

% Compute best fitting theta0, together with Eta0 and V0
p0 = length(theta00);
theta0 = lsqnonlin(@(theta) resid0(theta, eta0, x, meany'), theta00, [], [], options); 
Eta0 = eta0(x, theta0)'; % vector of eta0 values
Eta0prime = eta0prime(x, theta0);  % matrix of derivatives of eta2
[Q0,R0] = qr(Eta0prime);
V0 = Q0(:,(p0+1):N);


Etad = Eta1 - Eta0;
check = norm(Etad) - (tau1 + tau0);

%% Set an initial design

xi = zeros(N,1);
init_indices = round(linspace(1,N,n));
xi(init_indices) = 1;    
xi = xi/n;

%% Initialize for the loop over k
oldcount = 0;
bestdiv = 0;
iterationsWithoutImprovement = 0;

%% Start the loop over k
for k = 1:K 
    hundreds = floor(k/100);
    if hundreds > oldcount
        [hundreds, bestdiv]
        oldcount = oldcount + 1;
    end 

%% Obtain the minimizing deltas
    indPos = indices(xi>0)';
    s = length(indPos);
    if k==1 
        delta1start = zeros(s,1);
        delta0start = zeros(s,1);
        deltastart = [delta1start; delta0start];
    else
        delta1start = delta1(indPos);
        delta0start = delta0(indPos); 
        deltastart = [delta1start; delta0start];
    end

    [delta0, delta1] = Delta(xi, N, Eta1, Eta0, V1, V0, tau1, tau0, deltastart);

%% Obtain the next design
    mu1 = mu(Eta1, delta1);
    mu0 = mu(Eta0, delta0);
    psi = div(mu1, mu0);
    D(k) = xi'*psi;  %% Calculate D immediately after the min step
    Dmat(k,:) = [k, iterationsWithoutImprovement, D(k), bestdiv];
%    pause(1)
    if k > 1 & D(k) > bestdiv + 1e-8
        bestxi = xi;
        bestdiv = D(k);
        bestpsi = psi;
%        worstDeltas = [delta0, delta1];
        iterationsWithoutImprovement = 0; 
    end
    
	if alg == 1    
        [Ypsi,Ipsi] = sort(psi,'descend');
        dxi = zeros(N,1);
        dxi(Ipsi(1))=1;
        p = (k+1)^(-1); 
        xi = (1-p)*xi + p*dxi; 
        indPos = indices(xi>0)';
        [Y2,I2] = sort(psi(indPos), 'descend');
            if length(indPos) >= n + 1 
                xi(indPos(I2((n+1):length(indPos)))) = 0;  
            end
        xi = xi/sum(xi);    
	elseif  alg == 2        
        nxi = n*xi;
        indPos = indices(xi>0)';
        indextodecrease = indPos(psi(indPos) == min(psi(indPos)));
        indextoincrease = indices(psi == max(psi));
        nxi(indextodecrease) = nxi(indextodecrease) - 1;
        nxi(indextoincrease) = nxi(indextoincrease) + 1;
        xi = nxi/n;
	elseif alg == 3 
        xi = (psi - min(psi))/sum(psi - min(psi));
        xi = Round(xi, n, psi);
    end        
	iterationsWithoutImprovement = iterationsWithoutImprovement + 1;  
    
%% End the loop over k
end

%% Round the design

xi = bestxi;
bestDesignBeforeRounding = [x(xi>0); (xi(xi>0))'];
psi = bestpsi;
xi = Round(xi, n, psi); % Makes no difference if alg = 2
design = [x(xi>0); (xi(xi>0))'];

%% Do one more minimization over delta

indPos = indices(xi>0)';
delta1start = delta1(indPos);
delta0start = delta0(indPos); 
deltastart = [delta1start; delta0start];

[delta0, delta1] = Delta(xi, N, Eta1, Eta0, V1, V0, tau1, tau0, deltastart);

mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
psi = div(mu1, mu0);
%deltadiff = norm([delta0, delta1] - worstDeltas)
%psidiff = norm(psi - bestpsi) 
Dfinal = xi'*psi;
cvsqd = [1, .5, .1, .01];
psi0 = div(Eta1,Eta0);
maxpsi0 = max(psi0);

for j=1:length(cvsqd)
    multiple(j) = 1/(2*log(1 + cvsqd(j)))
    Power(j) = normcdf(sqrt(2*n*Dfinal*multiple(j))-norminv(.9));
	maxPower(j) = normcdf(sqrt(2*n*maxpsi0*multiple(j))-norminv(.9));
end

%% Plot the design

subplot(1,2,1); 
plot(x,psi);
axis([S(1) (S(2) + .1) 0 max(psi)]);
xlabel('(a)');
ylabel('psi');
subplot(1,2,2); 
bar(x,n*xi);
axis([S(1) (S(2) + .1) 0 max(n*xi)+1]);
xlabel('(b)');
ylabel('freq');  

%%
%% Subfunctions follow:
%%
%% ============================= subfunction "mm" =============================
function mm = mm(x, theta) 
V = theta(1);
K = theta(2);
mm = V*x./(K+x);

%% ============================= subfunction "expo" =============================
function expo = expo(x, theta) 
V = theta(1);
K = theta(2);
expo = V*(1-exp(-K*x)); 


%% ============================= subfunction "mmprime" =============================
function mmprime = mmprime(x, theta) 
V = theta(1);
K = theta(2);
mmprime = [x./(K+x); -V*x./((K+x).^2)]';

%% ============================= subfunction "expoprime" =============================
function expoprime = expoprime(x, theta) 
V = theta(1);
K = theta(2);
expoprime = [1-exp(-K*x); V*x.*exp(-K*x)]'; 

%% ============================= subfunction "resid1" =============================
function resid1 = resid1(theta, eta1, x, meany) 
resid1 = meany - eta1(x, theta);

%% ============================= subfunction "resid0" =============================
function resid0 = resid0(theta, eta0, x, meany) 
resid0 = meany - eta0(x, theta);


%% ============================= subfunction "mu" =============================
function mu = mu(Eta, delta) 
if min(Eta + delta <0)
   warning('log of negative number')
end
mu = max(1e-5, Eta + delta);
%mu = Eta + delta;

%% ============================= subfunction "div" =============================
function div = div(mu1, mu0)
%div = (mu1 - mu0).^2;
div = (log(mu1./mu0)).^2;

%% ============================= subfunction "P" =============================
function [P, g] = P(delta, xi, Eta1, Eta0)
s = length(xi);
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
P = xi'*div(mu1, mu0);

if nargout > 1
    qwe1 = (log(mu1./mu0))./mu1;
    qwe0 = (log(mu1./mu0))./mu0;
    g1 = 2*qwe1.*xi;
    g0 = -2*qwe0.*xi;
    g = [g1; g0];
end

%% ============================= subfunction "Pcon" =============================
function [c, ceq, GC, GCeq] = Pcon(delta, V1, V0, tau1, tau0)
ceq = 0;
s = length(delta)/2;
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
c = [delta1'*((V1*V1')\delta1)-tau1^2;delta0'*((V0*V0')\delta0)-tau0^2];
if nargout > 2
    GC = [2*((V1*V1')\delta1), zeros(s,1); zeros(s,1), 2*((V0*V0')\delta0)];
    GCeq = zeros(2*s,1);
end

%% ============================= subfunction "Delta" =============================
function [delta0, delta1] = Delta(xi, N, Eta1, Eta0, V1, V0, tau1, tau0, deltastart)
% Isolate the terms with xi>0
indices = 1:N;
indPos = indices(xi>0)';
indZero = indices(xi==0)';
Perm = eye(N);
Perm = Perm([indPos;indZero],:);    
    %Perm*v permutes v, putting Pos terms first. 
	%The transpose Perm' returns the permuted vector to original order
s = length(indPos);

% These term "...Pos" are the upper blocks, corresponding to xi>0:
xiPos = xi(indPos);
Eta1Pos = Eta1(indPos);
Eta0Pos = Eta0(indPos);
V1Pos = V1(indPos,:);
V0Pos = V0(indPos,:);

% These term "...Zero" are the upper blocks, corresponding to xi=0:
V1Zero = V1(indZero,:);
V0Zero = V0(indZero,:);

% Do the minimization over delta

options=optimset('Display','off','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
[delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos), ...
  deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

delta1Pos = delta(1:s);
delta0Pos = delta((s+1):(2*s));
    
c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

delta1Zero = V1Zero*c1star;
delta0Zero = V0Zero*c0star;

% Return the elements of delta to their natural order
delta1 = Perm'*[delta1Pos; delta1Zero];
delta0 = Perm'*[delta0Pos; delta0Zero];

  
%% ============================= subfunction "Round" =============================
function X = Round(xi, n, psi)
X = floor(n*xi);
indices = 1:length(X);
excess = n - sum(X);
if excess > 0
    indicesOfPosX = indices(X>0);
    Xpos = X(X>0);
    psiX = psi(X>0);
    [psiordered,I] = sort(psiX, 'descend');
	Xordered = Xpos(I);
    Xordered = Xordered + ...
        round(excess*(psiordered - min(psiordered))/...
                   sum(psiordered - min(psiordered)));
    X(indicesOfPosX(I)) = Xordered;           
end

newexcess = n - sum(X);
if newexcess > 0
    [Y,I] = sort(psi, 'descend');
	X(I(1:newexcess)) = X(I(1:newexcess)) + 1;
end

deficiency = sum(X) - n;
if deficiency > 0
    indicesOfPosX = indices(X>0);
    Xpos = X(X>0);
    psipos = psi(X>0);
    [Z,I] = sort(psipos, 'ascend');
	Xpos(I(1:deficiency)) = Xpos(I(1:deficiency)) - 1;
    X(indicesOfPosX(I)) = Xpos; 
end

X = X/n;  

