function out = KLstepwise(Tau, nInit, nFinal, maxheight)

% out = KLstepwise([.01,.05], 2, [20], [8,3]);

x = linspace(.1,5,50); %Set the design space
cvsqd = [1, .5, .1, .01];


for j = 1:length(nFinal)
    for k = 1:length(Tau)
        counter = j+k-(j==1);
        tau = Tau(k);
        [Theta1hat, Theta0hat, check, xi, design, Dfinal, Eta1, Eta0] = KL(x, tau, nInit, nFinal, 1);
        out(counter).Des = design;
        out(counter).check = check;
        out(counter).T1 = Theta1hat;
        out(counter).T0 = Theta0hat;
        subplot(length(nFinal),length(Tau),counter)
        bar(x,nFinal(j)*xi);
        axis([0 5.1 0 maxheight(k)]);
        if j==1 && k==1
            xlabel ('(a)');
        elseif j==1 && k==2
            xlabel ('(b)');	
        elseif j==2 && k==1
            xlabel ('(c)');
        elseif j==2 && k==2
            xlabel ('(d)');	

        end
        psi0 = div(Eta1,Eta0);
        maxpsi0 = max(psi0);
        for i=1:length(cvsqd)
            multiple(i) = 1/(2*log(1 + cvsqd(i)));
            Power(i) = normcdf(sqrt(2*nFinal*Dfinal*multiple(i))-norminv(.9));
            maxPower(i) = normcdf(sqrt(2*nFinal*maxpsi0*multiple(i))-norminv(.9));
        end
        out(counter).Power = Power;
        out(counter).maxPower = maxPower;
    end
end



%% ============================= subfunction "KL" =============================
function [theta1, theta0, check, xi, design, Dfinal, Eta1, Eta0] = KL(x, tau, nInit, nFinal, robust)

N = length(x);
indices = 1:N;

tau1 = tau;
tau0 = tau;

%% Set up those terms which are only computed once:

theta10 = [1;1];
theta00 = [1;1];

% Simulate E(Y|x)

X = [ones(N,1),x'];
lincoefs = regress(eta0(x,theta00)',X);
meany = X*lincoefs;
%figure; plot(x, meany, 'k', x, eta0(x,theta00)', ':', x, eta1(x,theta10)','--'); keyboard;
%meany = eta1(x, theta10)';

% Compute best fitting theta1, together with Eta1 and V1
options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
p1 = length(theta10);
theta1 = lsqnonlin(@(theta) resid1(theta, x, meany'), theta10, [], [], options); 
Eta1 = eta1(x, theta1)'; % vector of eta1 values
Eta1prime = eta1prime(x, theta1); % matrix of derivatives of eta1
[Q1,R1] = qr(Eta1prime);
V1 = Q1(:,(p1+1):N);

% Compute best fitting theta0, together with Eta2 and V2
p0 = length(theta00);
theta0 = lsqnonlin(@(theta) resid0(theta, x, meany'), theta00, [], [], options); 
Eta0 = eta0(x, theta0)'; % vector of eta0 values
Eta0prime = eta0prime(x, theta0);  % matrix of derivatives of eta2
[Q0,R0] = qr(Eta0prime);
V0 = Q0(:,(p0+1):N);


Etad = Eta1 - Eta0;
check = norm(Etad) - (tau1 + tau0);
%% Set an initial design

%xi = zeros(N,1);
%x1 = theta00(2)*x(N)/(x(N)+2*theta00(2));
%increm = (x(N)-x(1))/(N-1);
%index1 = 1+round((x1-x(1))/increm);  %%% CHECK THIS
%xi(index1) = nInit-floor(nInit/2);
%xi(N) = floor(nInit/2);

%%%%%%%%
xi = zeros(N,1);
x1 = .1;
x2 = 1.4;
increm = (x(N)-x(1))/(N-1);
index1 = 1+round((x1-x(1))/increm);  %%% CHECK THIS
xi(index1) = nInit-floor(nInit/2);
index2 = 1+round((x2-x(1))/increm);  %%% CHECK THIS
xi(index2) = nInit-floor(nInit/2);
%%%%%%%%

n = sum(xi)
xi = xi/n;

%% Start the loop over design size
while n < nFinal
    
%% Isolate the terms with xi>0
indPos = indices(xi>0)';
indZero = indices(xi==0)';
Perm = eye(N);
Perm = Perm([indPos;indZero],:);    
    %Perm*v permutes v, putting Pos terms first. 
	%The transpose Perm' returns the permuted vector to original order
s = length(indPos);

% These term "...Pos" are the upper blocks, corresponding to xi>0:
xiPos = xi(indPos);
Eta1Pos = Eta1(indPos);
Eta0Pos = Eta0(indPos);
V1Pos = V1(indPos,:);
V0Pos = V0(indPos,:);

% These term "...Zero" are the upper blocks, corresponding to xi=0:
V1Zero = V1(indZero,:);
V0Zero = V0(indZero,:);

%% Do the minimization over delta

if n > nInit 
    newIndex = indices(obj==max(obj));
    delta1(newIndex) = 0;
    delta1start = delta1(indPos);
    delta0(newIndex) = 0;
    delta0start = delta0(indPos);    
else
    delta1start = zeros(s,1);
    delta0start = zeros(s,1);
end
deltastart = [delta1start; delta0start];

if robust==1
    options=optimset('Display','final','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
    [delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos), ...
        deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

    delta1Pos = delta(1:s);
    delta0Pos = delta((s+1):(2*s));
else
	delta1Pos = zeros(s,1);
    delta0Pos = zeros(s,1);
end
 
c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

delta1Zero = V1Zero*c1star;
delta0Zero = V0Zero*c0star;

%% Return the elements of delta to their natural order
delta1 = Perm'*[delta1Pos; delta1Zero];
delta0 = Perm'*[delta0Pos; delta0Zero];

%% Obtain the next design
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
obj = div(mu1, mu0);

%plot(x,obj); keyboard

xi = n*xi + (obj==max(obj));
nextDesignPoint = x(obj==max(obj))
n = sum(xi)
xi = xi/n;

%% End the loop over design size
end

%% Do one more minimization

if n > nInit 
    newIndex = indices(obj==max(obj));
    delta1(newIndex) = 0;
    delta1start = delta1(indPos);
    delta0(newIndex) = 0;
    delta0start = delta0(indPos);    
else
    delta1start = zeros(s,1);
    delta0start = zeros(s,1);
end
deltastart = [delta1start; delta0start];

if robust==1
    options=optimset('Display','final','GradObj','on', 'GradConstr','on', 'LargeScale', 'off');
    [delta, fval, exitflag, output, lambda, grad] = fmincon(@(delta)P(delta, xiPos, Eta1Pos, Eta0Pos), ...
        deltastart, [],[],[],[],[],[],@(delta)Pcon(delta, V1Pos, V0Pos, tau1, tau0),options);

    delta1Pos = delta(1:s);
    delta0Pos = delta((s+1):(2*s));
else
	delta1Pos = zeros(s,1);
    delta0Pos = zeros(s,1);
end
 
c1star = V1Pos'*((V1Pos*V1Pos')\delta1Pos);
c0star = V0Pos'*((V0Pos*V0Pos')\delta0Pos);

delta1Zero = V1Zero*c1star;
delta0Zero = V0Zero*c0star;

%% Return the elements of delta to their natural order
delta1 = Perm'*[delta1Pos; delta1Zero];
delta0 = Perm'*[delta0Pos; delta0Zero];

mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
obj = div(mu1, mu0);

%% Summarize the design
design = [x(xi>0); (xi(xi>0))'];
Dfinal = xi'*obj; 

%%
%% Subfunctions follow:
%%
%% ============================= subfunction "eta1" =============================
function eta1 = eta1(x, theta) 
V = theta(1);
K = theta(2);
eta1 = V*(1-exp(-K*x)); % Expo
%eta1 = V*x./(K+x); % MM
%% ============================= subfunction "eta1prime" =============================
function eta1prime = eta1prime(x, theta) 
V = theta(1);
K = theta(2);
eta1prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
%eta1prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM
%% ============================= subfunction "resid1" =============================
function resid1 = resid1(theta, x, meany) 
resid1 = meany - eta1(x, theta);

%% ============================= subfunction "eta0" =============================
function eta0 = eta0(x, theta) 
V = theta(1);
K = theta(2);
%eta0 = V*(1-exp(-K*x)); % Expo
eta0 = V*x./(K+x); % MM
%% ============================= subfunction "eta0prime" =============================
function eta0prime = eta0prime(x, theta) 
V = theta(1);
K = theta(2);
%eta0prime = [1-exp(-K*x); V*x.*exp(-K*x)]'; % Expo
eta0prime = [x./(K+x); -V*x./((K+x).^2)]'; % MM
%% ============================= subfunction "resid0" =============================
function resid0 = resid0(theta, x, meany) 
resid0 = meany - eta0(x, theta);

%% ============================= subfunction "mu" =============================
function mu = mu(Eta, delta) 
if min(Eta + delta <0)
   warning('log of negative number')
end
mu = max(1e-5, Eta + delta);
%mu = Eta + delta;

%% ============================= subfunction "div" =============================
function div = div(mu1, mu0)
div = (log(mu1./mu0)).^2;

%% ============================= subfunction "P" =============================
function [P, g] = P(delta, xi, Eta1, Eta0)
s = length(xi);
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
mu1 = mu(Eta1, delta1);
mu0 = mu(Eta0, delta0);
P = xi'*div(mu1, mu0);

if nargout > 1
    qwe1 = (log(mu1./mu0))./mu1;
    qwe0 = (log(mu1./mu0))./mu0;
    g1 = 2*qwe1.*xi;
    g0 = -2*qwe0.*xi;
    g = [g1; g0];
end

%% ============================= subfunction "Pcon" =============================
function [c, ceq, GC, GCeq] = Pcon(delta, V1, V0, tau1, tau0)
ceq = 0;
s = length(delta)/2;
delta1 = delta(1:s);
delta0 = delta((s+1):(2*s));
c = [delta1'*((V1*V1')\delta1)-tau1^2;delta0'*((V0*V0')\delta0)-tau0^2];
if nargout > 2
    GC = [2*((V1*V1')\delta1), zeros(s,1); zeros(s,1), 2*((V0*V0')\delta0)];
    GCeq = zeros(2*s,1);
end
  
    
